/*
** CONFIG.C
**
** Configuracion.
**
**
** (c) 1.997-99 Jesus Cea Avion
** jcea@argo.es
** http://www.argo.es/~jcea/
** http://www.argo.es/~jcea/artic/radius02.htm
**
** Lea las condiciones de uso y distribucion
** en el fichero "COPYING".
**
**
** 14/Dic/98 Version 1.0
**           Este fichero permite tener en un unico
**           lugar, pero fuera de la libreria, la
**           la configuracion necesaria para todos
**           los programas que usan la libreria en si.
**
** 17/Dic/98 Soporte para modulo de salida generico.
**
** 30/Dic/98 Se continua la migracion al modulo
**           generico de salida.
**
** 04/Dic/98 Se cambian de sitio las rutinas "str2tiempo"
**           y "tiempo2str".
*/

#include "lee_log.h"
#include "lee_radius.h"
#include "lee_mrtg.h"
#include "config.h"
#include "salida.h"
#include "salida_html.h"
#include <stdlib.h>
#include <stdio.h>
#include <string.h>


#define MAX_TIEMPO  1893452400ul /* 01/01/2030 */
time_t tiempo_inicial=0;
time_t tiempo_final=MAX_TIEMPO;

/*
** Clase C. Ojo con el punto final.
** Por ejemplo, "192.168.1."
*/

char *RED[]={
  "PON_TU_SERIE_DE_CLASES_C",
  NULL};

/* Tabla para asignar usuarios fijos a determinadas IPs fijas */

t_ip_usuario IP_USUARIO[]={
  {NULL,NULL}};

/* La clave a usar en el CGI */

char *CLAVE="TU_CLAVE_DE_ACCESO";

/* Directorio donde estan los archivos historicos */

char *D_HISTORICO="PON_EL_PATH_A_LOS_ARCHIVOS_HISTORICOS/";

/* STUBS para rutinas de lectura de registros */

static t_log RADIUS = {
  abre_log_detalle_radius,
  cierra_log_detalle_radius,
  lee_registro_detalle_radius
};

log_estado lee_registro_detalle_radius_infovia_plus(registro_log *registro);
static t_log RADIUS_IPLUS = {
  abre_log_detalle_radius,
  cierra_log_detalle_radius,
  lee_registro_detalle_radius_infovia_plus
};

static t_log MRTG = {
  abre_log_mrtg,
  cierra_log_mrtg,
  lee_registro_mrtg
};

/*
** Define las rutinas que se van a utilizar para la salida
*/
t_salida SALIDA = {
  salida_cabecera_html,
  salida_conexiones_html,
  salida_final_html
};

/*
** Ficheros de detalle del RADIUS.
** El final se marca con NULL.
*/

char *ISP[]={
  "@PON_TU_CPI_UNO_SI_TIENE_ALIAS",
  "@PON_TU_CPI_DOS_SI_TIENE_ALIAS",
  "@PON_TU_CPI_TRES_SI_TIENE_ALIAS",
  "@PON_TU_CPI_CUATRO_SI_TIENE_ALIAS",
  NULL};


t_accounting accounting[]={
  {"TU_SERVIDOR_DE_TUNELES",ISP,&RADIUS_IPLUS   /* Tuneles */   },
  {"/export/home/snmp/",NULL,&MRTG  /* Estadisticas MRTG */ },
  {NULL,NULL,NULL}}; /* Marcador final */





/*
** Rutinas generales
*/



/*
** Como el servidor de tuneles de Infovia Plus
** no indica el Caller ID, pues lo indicamos
*/
log_estado lee_registro_detalle_radius_infovia_plus(registro_log *registro)
{
log_estado estado;

  estado=RADIUS.lee_registro(registro);
  if(registro->caller_id[0]=='\0') strcpy(registro->caller_id,"Infov&iacute;a Plus");
  return estado;
}

/*
** Rutina de apoyo para la siguiente.
*/
time_t str2tiempo2(char *tiempo)
{
char buf[32];
char c;
struct tm tm;

/*
  strptime(tiempo,"%Y%m%d%H%M",&tm);
*/

  strncpy(buf,tiempo,30);
  buf[30]='\0';
  c=buf[4]; buf[4]='\0';
  tm.tm_year=atoi(buf+0)-1900;
  buf[4]=c; c=buf[6]; buf[6]='\0';
  tm.tm_mon=atoi(buf+4)-1;
  buf[6]=c; c=buf[8]; buf[8]='\0';
  tm.tm_mday=atoi(buf+6);
  buf[8]=c; c=buf[10]; buf[10]='\0';
  tm.tm_hour=atoi(buf+8);
  buf[10]=c; c=buf[12]; buf[12]='\0';
  tm.tm_min=atoi(buf+10);

  tm.tm_sec=0; 
  tm.tm_isdst=-1;
  
  return mktime(&tm);
}

/*
** El tiempo convertido es local, no UTC
** Gestiona ?correctamente? el Daylight Saving Time.
*/
time_t str2tiempo(char *tiempo)
{
time_t t;
char buf[64];

  t=str2tiempo2(tiempo);
  tiempo2str(buf,t);
  return t+(t-str2tiempo2(buf));
}

void tiempo2str(char *buf,time_t tiempo)
{
struct tm *tm;

  tm=localtime(&tiempo);
  strftime(buf,CAMPO_LONG,"%Y%m%d%H%M",tm);
}

/*
** Esta rutina da el tiempo en segundos del
** primer segundo del mes cuyo tiempo se indica
*/
time_t dia1_este_mes(time_t tiempo)
{
char buf[64];
long fecha;

  tiempo2str(buf,tiempo);
  buf[6]='\0';
  fecha=atol(buf);
  sprintf(buf,"%lu010000",fecha);
  return str2tiempo(buf);
}

/*
** Esta rutina da el tiempo en segundos del
** primer segundo del mes SIGUIENTE al tiempo
** que se indica.
*/
time_t dia1_siguiente_mes(time_t tiempo)
{
time_t t,t2;

  tiempo=t=t2=dia1_este_mes(tiempo);
  do {
    t2+=25*(86400ul); /* Le sumo unos dias, hasta que cambiemos de mes */ 
    t=dia1_este_mes(t2);
  } while(tiempo==t);

  return t;
}

/*
** clipping_mensual
** Esta rutina devuelve un valor en segundos que
** se corresponde con:
** - El principio del mes en curso
** - El principio del mes anterior si el mes en
**   curso tiene menos de cuatro horas de vida.
*/
time_t clipping_mensual(void)
{
char buf[CAMPO_LONG];
unsigned long i;

  tiempo2str(buf,time(NULL));
  if((buf[6]=='0')&&(buf[7]=='1')&&
    (buf[8]=='0')&&(buf[9]<'4')) /* 4 horas */
  {
    buf[6]='\0';
    i=atol(buf);
    i-=1;
    if(!(i%100)) i-=88; /* Quita uno al ano y pone mes doce */
    sprintf(buf,"%lu010000",i);
  }
  else
  {
    strcpy(buf+6,"010000"); /* Desde el principio del mes */
  }
  return str2tiempo(buf);
}

void activa_clipping_mensual(int num_tiempos,char *tiempos[])
{
/*
char buf1[50],buf2[50];
*/

  if(num_tiempos<1) /* No se indican los tiempos */
  {
    tiempo_inicial=clipping_mensual();
  }
  else
  {
    tiempo_inicial=str2tiempo(tiempos[0]);
    if(num_tiempos>1) tiempo_final=str2tiempo(tiempos[1]);
  }

/*
  tiempo2str(buf1,tiempo_inicial);
  tiempo2str(buf2,tiempo_final);
  printf("Tiempo Inicial: %s\nTiempo Final: %s\n",buf1,buf2);
*/
}

