/*
** LEE_MRTG.C
**
** Lee logs de MRTG.
**
**
** (c) 1.997-99 Jesus Cea Avion
** jcea@argo.es
** http://www.argo.es/~jcea/
** http://www.argo.es/~jcea/artic/radius02.htm
**
** Lea las condiciones de uso y distribucion
** en el fichero "COPYING".
**
**
** 20/Nov/98 Primera version de este fichero.
**
** 23/Nov/98 Pasamos de vuelta los tiempos en segundos.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "lee_mrtg.h"

#include "util.h"

#undef strchr(a,b)
#define strchr(a,b) strchr_jcea(a,b)
#undef strcmp(a,b)
#define strcmp(a,b) strcmp_jcea(a,b)


/*
** El fichero de log del MRTG 2.x tiene 2533 lineas:
** - La primera contiene el epoch, in y out de la ultima ejecucion
** - La siguientes tienen el epoch, in/out RATE medio desde la ultima lectura
**   y in/out RATE de pico desde la ultima lectura. 
** Lo del RATE es el trafico en bytes dividido por 300 segundos.
** http://www.ee.ethz.ch/~slist/mrtg/msg01960.html
*/

typedef struct {
  time_t tiempo;
  time_t long_tiempo;
  unsigned long in;
  unsigned long out;
} t_mrtg;

static t_mrtg *mrtg=NULL,*mrtg_actual=NULL;
static int num_lineas=0;

#define RATE    300  /* Segundos por muestreo (en las primeras lineas) */
#define LINEAS  2532  /* La primera linea no cuenta */
#define MRTG_SIZE       LINEAS*sizeof(t_mrtg)

static registro_log reg_dummy;


void abre_log_mrtg(char *f_detalle,char *user)
{
static char buf[2*CAMPO_LONG];
char *p=user;
time_t long_tiempo=RATE;

  if(strlen(f_detalle)+strlen(user)>2*CAMPO_LONG-4) return;

  inicializa_registro(&reg_dummy);
  reg_dummy.tipo=STOP;
  strcpy(reg_dummy.nombre,user);
  strcpy(reg_dummy.caller_id,user);
  reg_dummy.causa=1; /* Normal */
  
  num_lineas=0;
  mrtg=mrtg_actual=malloc(MRTG_SIZE);
  if(mrtg==NULL) return;
  memset(mrtg,0,MRTG_SIZE);

  while(*p) /* Esto es por si alguien intenta algo raro... */
  {
    if(!isalnum(*p)&&(*p!='-')&&(*p!='_')&&(*p!='.')) return;
    p++;
  }
  strcpy(buf,f_detalle);
  strcat(buf,user);
  strcat(buf,".log");
  abre_log_cache(buf);
  lee_log_cache(buf); /* La primera linea la ignoramos */
  while(lee_log_cache(buf)==LOG_OK)
  {
    mrtg_actual->tiempo=atol(buf);
    mrtg_actual->long_tiempo=long_tiempo;
    p=strrchr(buf,' ');
    if(p==NULL) break;
    mrtg_actual->out=long_tiempo*atol(p+1);
    *p--='\0';
    p=strrchr(buf,' ');
    if(p==NULL) break;
    mrtg_actual->in=long_tiempo*atol(p+1);
    mrtg_actual++;
    switch(++num_lineas)
    {
    case 600:
      long_tiempo*=6;  /* Promedio de 30 minutes */
      break;
    case 1200:
      long_tiempo*=4;  /* Promedio de 2 horas */
      break;
    case 1800:
      long_tiempo*=12; /* Promedio de 24 horas */
      break;
    case LINEAS:  /* El fichero deberia terminar aqui */
      goto SALIR; /* Por si las moscas... */
      break;
    }
  }
SALIR:
  mrtg_actual--; /* Apunta a lo ultimo que hemos leido */
  cierra_log_cache();
}

void cierra_log_mrtg(void)
{
  cierra_log_cache();
  if(mrtg) free(mrtg);
  mrtg=mrtg_actual=NULL;
  num_lineas=0;
}

/*
** Antigua rutina
*/
/*****
log_estado lee_registro_mrtg(registro_log *registro)
{
  *registro=reg_dummy;
  registro->tiempo=mrtg_actual->tiempo;
  registro->duracion=mrtg_actual->long_tiempo;
  sprintf(registro->sesion_id,"%d",num_lineas);
  if(!(num_lineas--)) return LOG_EOF;
  registro->bytes_tx=mrtg_actual->in;
  registro->bytes_rx=mrtg_actual->out;
  mrtg_actual--;
  return LOG_OK;
}
*****/

log_estado lee_registro_mrtg(registro_log *registro)
{
  *registro=reg_dummy;
  while(num_lineas--) {
    if((mrtg_actual->in==0)&&(mrtg_actual->out==0)) {  /* Eliminamos los registros vacios */
      mrtg_actual--;
      continue;
    }
    registro->tiempo=mrtg_actual->tiempo;
    registro->duracion=mrtg_actual->long_tiempo;
    sprintf(registro->sesion_id,"%d",num_lineas+1);
    registro->bytes_tx=mrtg_actual->in;
    registro->bytes_rx=mrtg_actual->out;
    mrtg_actual--;
    return LOG_OK;
  }
  return LOG_EOF;
}

