/*
** LEE_RADIUS.C
**
** Rutinas de acceso a los LOGs de Infova
**
**
** (c) 1.997-99 Jesus Cea Avion
** jcea@argo.es
** http://www.argo.es/~jcea/
** http://www.argo.es/~jcea/artic/radius02.htm
**
** Lea las condiciones de uso y distribucion
** en el fichero "COPYING".
**
**
** 23/Abr/97 Primera versin de este fichero.
**           Aadida cach de disco.
**           Aadidas llamadas para la apertura y cierre de logs
**           Si llega al final del fichero, lo cierra automticamente.
**
** 26/Jun/97 Aadidos CSIVs de Bilbao y Sevilla
**
** 18/Sep/97 Aadidos campos de trfico Entrada/Salida
**
** 28/Sep/98 Por un bug no leia el campo "Acct-Session-Id"
**
** 13/Oct/98 En vez de indicar el CSIV, se indica directamente
**           el Path del fichero de detalle.
**           Define la interfaz publica de este modulo.
**           Enorme mejora (X3) en el rendimiento, gracias a
**           - Trabajo con cache en memoria.
**           - Lectura linea a linea, no caracter a caracter.
**           Cambiamos los atoll(), que en Linux dan problemas,
**           por atol(). Esto afecta a los contadores de
**           bytes y paquetes.
**
** 20/Oct/98 Segregamos a este fichero las rutinas exclusivamente
**           de acceso a los ficheros de detalle del RADIUS.
**           Cambiando la estructura de las comparaciones,
**           logramos una mejora de tiempo global del 17%.
**           El uso de una libreria de cadenas propia mejora el rendimiento.
**
** 26/Oct/98 Rutina de busqueda de los
**           registros especificos de un usuario.
**
** 23/Nov/98 La fecha y hora de los registros es pasado
**           a segundos desde 1.970.
**           Reconoce las causas de desconexion en Acct-Terminate-Cause.
**           Optimizacion de la conversion de hora, haciendo cache del dia.
**
** 18/Ene/99 Acct-Delay-Time
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <time.h>
#include "lee_radius.h"
#include "config.h"

#include "util.h"

#undef strchr(a,b)
#define strchr(a,b) strchr_jcea(a,b)
#undef strcmp(a,b)
#define strcmp(a,b) strcmp_jcea(a,b)



static char usuario[CAMPO_LONG];
static char dia[CAMPO_LONG];
static long dia_en_segundos;

void abre_log_detalle_radius(char *f_detalle,char *user)
{
  if(strlen(user)>CAMPO_LONG-4) return;
  strcpy(usuario,user);
  abre_log_cache(f_detalle);
}

void cierra_log_detalle_radius(void)
{
  cierra_log_cache();
}

time_t radius2tiempo(char *tiempo)
{
char buf[64];

  memcpy(buf,tiempo+6,4);   /* An~o */
  memcpy(buf+4,tiempo+3,2); /* Mes */
  memcpy(buf+6,tiempo,2);   /* Dia */
  buf[8]='\0';
  strcat(buf,"0000");       /* Horas y minutos */
  return str2tiempo(buf);
}


log_estado lee_registro_detalle_radius(registro_log *registro)
{
char buf[CAMPO_LONG];
log_estado estado;
char *p,*p2;
// struct tm tm;
int retardo_accounting=0;

  inicializa_registro(registro);
  do {
    estado=lee_log_cache(buf);
    if(*buf=='\0') break; /* Terminamos con el registro y/o el fichero */

  /* ?Es fecha y hora? */
    if(*buf!='\t') {
      p=strchr(buf,' ');
      if(p==NULL) { /* Este campo no parece legal */
      /* No podemos abortar el programa
      ** debido a que puede tratarse de
      ** un fichero de log con timestamp.
      */
/*
        strptime(buf,"%d-%m-%Y %T",&tm);
        registro->tiempo=mktime(&tm);
*/
      } else {
        *p='\0';
        if(strcmp(buf,dia)) {
          strcpy(dia,buf);

        /*
        ** Esto esta mal porque no es
        ** correcto en el DST.
        */
/*
          strptime(buf,"%d-%m-%Y",&tm);
          dia_en_segundos=mktime(&tm);
*/
          dia_en_segundos=radius2tiempo(buf);
        }

        registro->tiempo=dia_en_segundos+(*(p+1)-'0')*36000+(*(p+2)-'0')*3600+
          (*(p+4)-'0')*600+(*(p+5)-'0')*60+
          (*(p+7)-'0')*10+(*(p+8)-'0');
        *p=' ';
      }
      continue; /* Lee linea siguiente */
    }
   p=strchr(buf,'=');
   if(p==NULL) return LOG_EOF; /* Linea cortada */
   *(p-1)='\0';
   p+=2;  /* Valor */
   if(*p=='\"') { /* Eliminamos las comillas */
     p++;
     *strchr(p,'\"')='\0';
   }
   p2=buf;
   while((*p2==' ')||(*p2=='\t')) p2++; /* Espacios iniciales */

   if(!strcmp("User-Name",p2))
   {
     if((*usuario!='\0') && strcmp(usuario,p))
     { /* No nos interesa este tio. Nos saltamos su ficha */
       do {
         estado=lee_log_cache(buf);
       } while(*buf!='\0');
       inicializa_registro(registro);
       continue; /* Inicia una ficha nueva */
     }
     strcpy(registro->nombre,p);
   }
   else if(!strncmp("Acct-",p2,5))
   {
     p2+=5;  
     if(!strcmp("Status-Type",p2))
     {
       if(!strcmp("Start",p)) registro->tipo=START;
       else if(!strcmp("Stop",p)) registro->tipo=STOP;
       else
       {
         perror("PaniC!");
         exit(0);
       }
     } else if(!strcmp("Session-Id",p2))
       strcpy(registro->sesion_id,p);
     else if(!strcmp("Session-Time",p2))
       registro->duracion=atol(p);
     else if(!strcmp("Output-Octets",p2))
       registro->bytes_rx=atol(p);
     else if(!strcmp("Input-Octets",p2))
       registro->bytes_tx=atol(p);
     else if(!strcmp("Output-Packets",p2))
       registro->paquetes_rx=atol(p);
     else if(!strcmp("Input-Packets",p2))
       registro->paquetes_tx=atol(p);
     else if(!strcmp("Terminate-Cause",p2))
       registro->causa=atoi(p);
     else if(!strcmp("Delay-Time",p2))
       retardo_accounting=atoi(p);
   }
   else if(!strcmp("Timestamp",p2))
     registro->tiempo=atol(p);
   else if(!strcmp("Caller-Id",p2))
     strcpy(registro->caller_id,p);
   else if(!strcmp("Calling-Station-Id",p2))
     strcpy(registro->caller_id,p);
   else if(!strcmp("Ascend-Disconnect-Cause",p2))
     registro->causa=1000+atoi(p);
   else if(!strcmp("Ascend-Data-Rate",p2))
     registro->velocidad=atol(p);
   else if(!strcmp("Connect-Info",p2))
     registro->velocidad=atol(p);
   else if(!strcmp("Framed-Address",p2))
     strcpy(registro->ip,p);
   else if(!strcmp("Framed-IP-Address",p2))
     strcpy(registro->ip,p);

  } while(estado==LOG_OK);
  registro->tiempo-=retardo_accounting;

/*
** Algunos NAS mandan ceros al final
*/
  if((*(registro->caller_id)>='0') && (*(registro->caller_id)<='9'))
    *(registro->caller_id+9)='\0';
 
  return estado;
}

